/* Copyright (c) 2021 Asustor Inc. All rights reserved. */

/**
 * @class AS.ARC.apps.dockerExtension.core
 * @extends Ext.util.Observable
 * <p>core</p>
 */
Ext.define('AS.ARC.apps.dockerExtension.core', {
	extend: 'Ext.util.Observable',

	constructor: function (config) {
		Ext.apply(this, config);
		this.callParent();
		this.init(config);
	},

	init: function () {
		var page = this;
		page.apiUrl = AS.ARC.util.getUserAppsPath() + page.app.appTag + '/setting.cgi';
		page.win = page.desktop.createWindow({
			app: page.app,
			id: page.id,
			itemId: page.id,
			width: 880,
			height: 560,
			border: false,
			layout: 'border',
			items: [page.renderMainPanel(), page.renderWestPanel()],
			listeners: {
				afterrender: function () {
					var panel = page.mainPanel;
					panel.statusTip.bindMethodWith(panel);
					panel.uiChecker.bindMethodWith(panel);
					page.loadData();
				},
			},
		});
	},
	overrideStatusTip: function () {
		var page = this;
		var statusTip = Ext.create('AS.ARC.statusTip');
		//override statusTip from portal/core/asComp.js: showMsgAfterApply
		statusTip.showMsgAfterApply = function (imgSrcHTML, valueHTML) {
			var controller = this,
				field = controller.tipField,
				footerDf = field,
				value = imgSrcHTML + ' ' + valueHTML;

			if (controller.tipField === null) {
				return false;
			}

			if (controller.idTimer !== null) {
				clearTimeout(controller.idTimer);
			}

			footerDf.hide();
			footerDf.setValue(value);
			footerDf.el.fadeIn({ useDisplay: true });
			footerDf.show();

			controller.idTimer = setTimeout(function () {
				if (typeof footerDf.el !== 'undefined') {
					footerDf.el.fadeOut({ useDisplay: false });
					if (Ext.isIE8 || Ext.isIE7 || Ext.isIE6) {
						footerDf.hide();
					}
				}
				controller.idTimer = null;
			}, 5000);
		};
		return statusTip;
	},
	handleApply: function () {
		var page = this;
		var params = page.mainPanel.doDataExchange();

		if (!page.mainPanel.isSettingChange()) {
			page.mainPanel.showSetNotChange();
		} else {
			var urlParams = {};
			var srcData = page.mainPanel.uiChecker.srcData;
			var mirrorURL = page.mainPanel.down('#registry-mirror');
			var callback = function () {
				page.mainPanel.showApply();
				page.loadData();
			};
			Ext.apply(urlParams, {
				url: params['url'],
			});
			var setUrlAndEnableMirror = function (callback) {
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, { act: 'set' }),
					method: 'post',
					params: urlParams,
					success: function (json, opts) {
						Ext.callback(callback);
					},
					failure: function (json, opts) {
						page.win.el.unmask();
						AS.ARC.util.showMsgWindow(
							{
								5401: _S('ERROR', 'INCORRECT_PARAMETER'),
							},
							json,
							page.win
						);
					},
				});
			};
			var enableMirror = function (callback) {
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, { act: 'enable' }),
					method: 'post',
					success: function (json, opts) {
						Ext.callback(callback);
					},
					failure: function (json, opts) {
						page.win.el.unmask();
						AS.ARC.util.showMsgWindow(
							{
								5401: _S('ERROR', 'UNABLE_APPLY_SETTINGS'),
							},
							json,
							page.win
						);
					},
				});
			};
			var disableMirror = function (callback) {
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, { act: 'disable' }),
					method: 'post',
					success: function (json, opts) {
						Ext.callback(callback);
					},
					failure: function (json, opts) {
						page.win.el.unmask();
						AS.ARC.util.showMsgWindow(
							{
								5401: _S('ERROR', 'UNABLE_APPLY_SETTINGS'),
							},
							json,
							page.win
						);
					},
				});
			};
			page.win.el.mask(_S('COMMON', 'APPLYING'));
			var urlNotEdited = function () {
				return srcData['url'] === params['url'];
			};
			var urlInvalid = function () {
				return mirrorURL.validator(params['url']) !== true;
			};
			if (urlNotEdited() || urlInvalid()) {
				params['enable'] ? enableMirror(callback) : disableMirror(callback);
			} else {
				params['enable']
					? setUrlAndEnableMirror(callback)
					: setUrlAndEnableMirror(function () {
							disableMirror(callback);
					  });
			}
		}
	},

	renderWestPanel: function () {
		var page = this;
		page.westPanel = Ext.create('Ext.panel.Panel', {
			cls: 'as-page-panel',
			page: page,
			region: 'west',
			width: 250,
			bodyStyle: 'border: none; background: transparent;',
			cls: 'docker-extension-left-banner',
			style: 'background-repeat: no-repeat;' + 'background-position-y: 100%;',
		});
		return page.westPanel;
	},
	renderMainPanel: function () {
		var page = this;
		var labelWidth = AS.ARC.config.layout.indentWidth * 2 + AS.ARC.config.layout.labelWidthSmall;
		var inputWidth = 250;
		var buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY'),
			handler: function () {
				page.handleApply();
			},
		});
		var statusTip = page.overrideStatusTip();
		var uiChecker = Ext.create('AS.ARC.uiModifyChecker');
		page.mainPanel = Ext.create('AS.ARC.panelTop20', {
			cls: 'as-page-panel',
			style: 'padding-top:36px; padding-bottom:15px',
			page: page,
			buttonApply: buttonApply,
			region: 'center',
			border: false,
			items: [
				{
					xtype: 'fieldset',
					itemId: 'fs-docker-extension',
					title: _TS_ING('DOCKER_EXTENSION', 'REGISTRY_SETTING'),
					defaults: {
						labelWidth: labelWidth,
						msgTarget: AS.ARC.config.msgTarget,
						blankText: _S('MESSAGE', 'HF00011'),
					},
					items: [
						{
							xtype: 'displayfield',
							value: _TS_ING('DOCKER_EXTENSION', 'DEFAULT_REGISTRY'),
						},
						{
							xtype: 'textfield',
							fieldLabel: AS.layout.getIndent(1, _S('COMMON', 'NAME')),
							value: 'Docker Hub',
							width: labelWidth + inputWidth,
							disabled: true,
						},
						{
							xtype: 'textfield',
							fieldLabel: AS.layout.getIndent(1, _TS_ING('DOCKER_EXTENSION', 'URL')),
							value: 'https://registry.hub.docker.com',
							width: labelWidth + inputWidth,
							disabled: true,
						},
						{
							xtype: 'checkbox',
							itemId: 'registry-enable',
							boxLabel: _TS_ING('DOCKER_EXTENSION', 'ENABLE_REGISTRY_MIRROR'),
							handler: function (checkbox, checked) {
								var mirror = page.mainPanel.down('#registry-mirror');
								mirror.setDisabled(!checked);
								mirror.clearInvalid();
								mirror.isValid();
							},
						},
						{
							xtype: 'textfield',
							itemId: 'registry-mirror',
							fieldLabel: AS.layout.getIndent(1, _TS_ING('DOCKER_EXTENSION', 'REGISTRY_MIRROR_URL')),
							width: labelWidth + inputWidth,
							disabled: true,
							validator: function (value) {
								var urlRe = /^https?:\/\/[A-Za-z0-9-_.~:/?#[\]@!$&'()*+,;=]+$/;
								var regex = new RegExp(urlRe);
								if (!value) {
									return _S('MESSAGE', 'HF00011');
								}
								if (!regex.match(value)) {
									return _S('MESSAGE', 'HF00032');
								}
								return true;
							},
						},
					],
				},
			],
			statusTip: statusTip,
			uiChecker: uiChecker,
			fbar: [statusTip.getTipField(), { xtype: 'tbfill' }, buttonApply],
			doDataExchange: function () {
				var enable = page.mainPanel.down('#registry-enable').getValue(),
					url = page.mainPanel.down('#registry-mirror').getValue();
				return {
					enable: enable,
					url: url,
				};
			},
		});
		return page.mainPanel;
	},
	loadData: function () {
		var page = this,
			panel = page.mainPanel;
		page.win.el.mask(_S('COMMON', 'LOADING'));
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, { act: 'get' }),
			method: 'post',
			success: function (json, opts) {
				panel.uiChecker.srcData = json;
				panel.down('#registry-enable').setValue(json['enable']);
				panel.down('#registry-mirror').setValue(json['url']);
				page.win.el.unmask();
			},
			failure: function (json, opts) {
				page.win.el.unmask();
				AS.ARC.util.showMsgWindow({}, json, page.win);
			},
		});
	},
});

/**
 * @class AS.ARC.apps.dockerExtension.main
 * @extends AS.ARC._appBase
 * <p>Main app</p>
 */
Ext.define('AS.ARC.apps.dockerExtension.main', {
	extend: 'AS.ARC._appBase',

	appTag: 'docker-ce-registry-pack',
	appMaxNum: 1,
	appOpenNum: 0,
	appIsReady: true,
	appWins: [],

	createWindow: function () {
		var desktop = this.core.getDesktop(),
			app = this;

		if (this.appOpenNum === this.appMaxNum || !this.appIsReady) {
			this.appWins[0].show();
			return;
		}

		this.appIsReady = false;

		var newDC = new AS.ARC.apps.dockerExtension.core({
			app: this,
			desktop: desktop,
			id: this.id + '-' + Ext.id(),
		});

		newDC.win.on('render', function () {
			app.appOpenNum++;
			app.appIsReady = true;
			app.appWins.pop();
		});

		newDC.win.on('beforeclose', function () {
			if (newDC.mainPanel.isSettingChange() && !newDC.win.readyToClose) {
				var confirmWin = Ext.create('AS.ARC.msgWindow', {
					title: _S('COMMON', 'CONFIRMATION'),
					parentWin: newDC.win,
					width: 400,
					iconType: 'question',
					asItems: [
						{
							xtype: 'displayfield',
							value: _S('COMMON', 'CONFIRM_SETTING_MODIFY'),
						},
					],
					fbar: [
						{
							text: _S('COMMON', 'OK'),
							itemId: 'apply',
							handler: function () {
								newDC.win.readyToClose = true;
								confirmWin.close();
								newDC.win.close();
							},
						},
						{
							text: _S('COMMON', 'CANCEL'),
							handler: function () {
								confirmWin.close();
							},
						},
					],
				}).show();
				return false;
			} else {
				app.appOpenNum--;
				app.appIsReady = true;
			}
		});

		newDC.win.show();
		// Get all wins for handling
		this.appWins.push(newDC.win);
	},
});
